@@ -18,6 +18,11 @@ class FailedToMoveVoiceCardException(Exception):
         return repr("Failed to move voice card.")
 
 
+class FailedToDeleteVoiceCardException(Exception):
+    def __str__(self):
+        return repr("Failed to delete voice card.")
+
+
 class FailedToSetModelDirException(Exception):
     def __str__(self):
         return repr("Failed set the directory for storing the voice models.")
