@@ -1,11 +1,10 @@
-import json
 import os
 import re
-import shutil
 from typing import Iterable, List
 
 from PySide6.QtCore import QModelIndex, QSettings, QSize, Qt, Signal
 from PySide6.QtGui import (
+    QAction,
     QDragEnterEvent,
     QDragMoveEvent,
     QDropEvent,
@@ -21,16 +20,21 @@ from PySide6.QtWidgets import (
     QListView,
     QListWidget,
     QListWidgetItem,
+    QMenu,
     QMessageBox,
     QPushButton,
     QVBoxLayout,
     QWidget,
 )
-from voiceconversion.utils.LoadModelParams import LoadModelParamFile, LoadModelParams
+from voiceconversion.utils.import_model_params import (
+    ImportModelParamFile,
+    ImportModelParams,
+)
 
-from .audiosettings import AudioSettingsGroupBox
 from .exceptions import FailedToMoveVoiceCardException
 from .modelsettings import ModelSettingsGroupBox
+from .processingsettings import ProcessingSettingsGroupBox
+from .voicecardsmanager import VoiceCardsManager
 
 VOICE_CARD_SIZE = QSize(188, 262)
 VOICE_CARD_MARGIN = 8
@@ -43,46 +47,30 @@ RUNNING_TXT = "Running..."
 PASS_THROUGH_TXT = "Pass Through"
 
 
-def voiceCardForSlot(modelDir: str, row: int) -> QWidget:
-    folder = str(row)
-    paramsFilePath = os.path.join(modelDir, folder, "params.json")
-    if os.path.exists(paramsFilePath):
-        name = UNKNOWN_MODEL_NAME
-        with open(paramsFilePath) as f:
-            params = json.load(f)
-            iconFileName = params.get("iconFile", "")
-            name = params.get("name", name)
-            if iconFileName:
-                pixmap = QPixmap(os.path.join(modelDir, folder, iconFileName))
-                basename = os.path.splitext(os.path.basename(iconFileName))[0]
-                label = QLabel()
-                label.setPixmap(cropCenterScalePixmap(pixmap, VOICE_CARD_SIZE))
-                label.setToolTip(basename)
-                return label
-    return VoiceCardPlaceholderWidget(
-        VOICE_CARD_SIZE, f"{name}<br><br>{DROP_ICON_FILE}"
-    )
-
-
 class WindowAreaWidget(QWidget):
-    def __init__(self, modelDir: str, parent: QWidget | None = None):
+    cardMoved = Signal(int, int)
+    cardsRemoved = Signal(int, int)
+
+    def __init__(
+        self,
+        voiceCardsManager: VoiceCardsManager,
+        parent: QWidget | None = None,
+    ):
         super().__init__(parent)
 
-        self.modelDir = modelDir
-
         settings = QSettings()
-        settings.beginGroup("Interface")
+        settings.beginGroup("InterfaceSettings")
 
         layout = QVBoxLayout()
 
-        self.voiceCards = VoiceCardsContainer(modelDir)
+        self.voiceCards = VoiceCardsContainer(voiceCardsManager)
 
         layout.addWidget(self.voiceCards, stretch=2)
 
         controlsLayout = QHBoxLayout()
 
-        self.audioSettingsGroupBox = AudioSettingsGroupBox()
-        controlsLayout.addWidget(self.audioSettingsGroupBox, stretch=3)
+        self.processingSettingsGroupBox = ProcessingSettingsGroupBox()
+        controlsLayout.addWidget(self.processingSettingsGroupBox, stretch=1)
 
         self.modelSettingsGroupBox = ModelSettingsGroupBox()
         controlsLayout.addWidget(self.modelSettingsGroupBox, stretch=1)
@@ -107,18 +95,9 @@ class WindowAreaWidget(QWidget):
                 RUNNING_TXT if checked else START_TXT
             )
         )
-        # Unfortunately can't drag the cards while the voice conversion is running
-        # because it will select them and load.
+        # Can't change processing settings while running.
         self.startButton.toggled.connect(
-            lambda checked: self.voiceCards.setDragDropMode(
-                QAbstractItemView.DragDropMode.DropOnly
-                if checked
-                else QAbstractItemView.DragDropMode.InternalMove
-            )
-        )
-        # Can't change audio settings while running.
-        self.startButton.toggled.connect(
-            lambda checked: self.audioSettingsGroupBox.setEnabled(not checked)
+            lambda checked: self.processingSettingsGroupBox.setEnabled(not checked)
         )
         buttonsLayout.addWidget(self.startButton)
 
@@ -139,16 +118,8 @@ class WindowAreaWidget(QWidget):
 
         self.setLayout(layout)
 
-        modelDirToModelIcon: dict[str, QWidget] = {}
-
-        os.makedirs(modelDir, exist_ok=True)
-
-        for folder in os.listdir(modelDir):
-            if folder.isdigit() and os.path.isdir(os.path.join(modelDir, folder)):
-                modelDirToModelIcon[folder] = voiceCardForSlot(modelDir, int(folder))
-
-        for folder in sortedNumerically(modelDirToModelIcon):
-            self.voiceCards.addWidget(modelDirToModelIcon[folder])
+        for voiceCardIndex in range(voiceCardsManager.count()):
+            self.voiceCards.addWidget(self.voiceCards.voiceCardForIndex(voiceCardIndex))
 
         self.voiceCards.addWidget(
             VoiceCardPlaceholderWidget(
@@ -157,7 +128,29 @@ class WindowAreaWidget(QWidget):
             selectable=False,
         )
 
-        self.voiceCards.model().rowsMoved.connect(self.rearrangeVoiceModelDirs)
+        self.voiceCards.model().rowsMoved.connect(self.onCardsMoved)
+        self.voiceCards.model().rowsRemoved.connect(self.onCardsRemoved)
+        self.voiceCards.model().rowsRemoved.connect(
+            lambda _, first, last: settings.setValue(
+                "currentVoiceCardIndex",
+                (
+                    last
+                    if settings.value("currentVoiceCardIndex", 0) > last
+                    else settings.value("currentVoiceCardIndex", 0)
+                ),
+            )
+        )
+
+        def onRowsInserted(_: QModelIndex, first: int, last: int):
+            if last - first + 1 == self.voiceCards.count() - 1:  # minus the placeholder
+                self.voiceCards.setCurrentRow(
+                    int(settings.value("currentVoiceCardIndex", 0))
+                )
+
+        # QueuedConnection to make sure that the widget is properly set into the card.
+        self.voiceCards.model().rowsInserted.connect(
+            onRowsInserted, type=Qt.ConnectionType.QueuedConnection
+        )
 
         self.voiceCards.setCurrentRow(int(settings.value("currentVoiceCardIndex", 0)))
         self.voiceCards.currentRowChanged.connect(
@@ -170,7 +163,7 @@ class WindowAreaWidget(QWidget):
         )
         self.startButton.setEnabled(self.voiceCards.currentRow() >= 0)
 
-    def rearrangeVoiceModelDirs(
+    def onCardsMoved(
         self,
         sourceParent: QModelIndex,
         sourceStart: int,
@@ -181,40 +174,10 @@ class WindowAreaWidget(QWidget):
         if sourceStart != sourceEnd:
             raise FailedToMoveVoiceCardException
 
-        dirs = sorted(
-            [d for d in os.listdir(self.modelDir) if d.isdigit()], key=lambda x: int(x)
-        )
-        total = len(dirs)
+        self.cardMoved.emit(sourceStart, destinationRow)
 
-        if not (0 <= sourceStart < total) or not (0 <= destinationRow <= total):
-            raise FailedToMoveVoiceCardException("Invalid indices")
-
-        if destinationRow > sourceStart:
-            destinationRow -= 1
-
-        # Create a temp name for the moving directory to avoid name conflicts
-        src_path = os.path.join(self.modelDir, str(sourceStart))
-        tmp_path = os.path.join(self.modelDir, "_tmp_move")
-        shutil.move(src_path, tmp_path)
-
-        # Renumber other directories depending on move direction
-        if sourceStart < destinationRow:
-            # Shift everything between (sourceStart+1 ... destinationRow) up by one
-            for i in range(sourceStart + 1, destinationRow + 1):
-                os.rename(
-                    os.path.join(self.modelDir, str(i)),
-                    os.path.join(self.modelDir, str(i - 1)),
-                )
-        else:
-            # Shift everything between (destinationRow ... sourceStart-1) down by one
-            for i in range(sourceStart - 1, destinationRow - 1, -1):
-                os.rename(
-                    os.path.join(self.modelDir, str(i)),
-                    os.path.join(self.modelDir, str(i + 1)),
-                )
-
-        # Move the temp folder into its new numbered slot
-        shutil.move(tmp_path, os.path.join(self.modelDir, str(destinationRow)))
+    def onCardsRemoved(self, parent: QModelIndex, first: int, last: int):
+        self.cardsRemoved.emit(first, last)
 
 
 class FlowContainer(QListWidget):
@@ -292,12 +255,12 @@ class FlowContainerWithFixedLast(FlowContainer):
 
 class VoiceCardsContainer(FlowContainerWithFixedLast):
 
-    droppedModelFiles = Signal(LoadModelParams)
+    droppedModelFiles = Signal(int, ImportModelParams)
     droppedIconFile = Signal(int, str)
 
-    def __init__(self, modelDir: str):
+    def __init__(self, voiceCardsManager: VoiceCardsManager):
         super().__init__()
-        self.modelDir = modelDir
+        self.voiceCardsManager = voiceCardsManager
 
     def dragEnterEvent(self, event: QDragEnterEvent):
         if event.mimeData().hasUrls():
@@ -357,35 +320,60 @@ class VoiceCardsContainer(FlowContainerWithFixedLast):
 
         row = self.indexAt(event.pos()).row()
         importingNew = row < 0 or row >= self.count()
-        slot = self.count() - 1 if importingNew else row
+        voiceCardIndex = self.count() - 1 if importingNew else row
 
         if len(indexFiles) == 1:
             self.droppedModelFiles.emit(
-                LoadModelParams(
-                    voiceChangerType="RVC",
-                    slot=slot,
-                    isSampleMode=False,
-                    sampleId="",
+                voiceCardIndex,
+                ImportModelParams(
+                    voice_changer_type="RVC",
                     files=[
-                        LoadModelParamFile(
+                        ImportModelParamFile(
                             name=pthFiles[0].toLocalFile(), kind="rvcModel", dir=""
                         ),
-                        LoadModelParamFile(
+                        ImportModelParamFile(
                             name=indexFiles[0].toLocalFile(), kind="rvcIndex", dir=""
                         ),
                     ],
                     params={},
-                )
+                ),
             )
 
-        if len(iconFiles) == 1 and slot < self.count() - 1:
-            self.droppedIconFile.emit(slot, iconFiles[0].toLocalFile())
+        if len(iconFiles) == 1 and voiceCardIndex < self.count() - 1:
+            self.droppedIconFile.emit(voiceCardIndex, iconFiles[0].toLocalFile())
 
     def onVoiceCardUpdated(self, row: int):
         if row >= self.count() - 1:
-            self.insertWidget(row, voiceCardForSlot(self.modelDir, row))
+            self.insertWidget(row, self.voiceCardForIndex(row))
         else:
-            self.setItemWidget(self.item(row), voiceCardForSlot(self.modelDir, row))
+            self.setItemWidget(self.item(row), self.voiceCardForIndex(row))
+
+    def voiceCardForIndex(self, row: int) -> QWidget:
+        widget: QWidget | QLabel | None = None
+        name = UNKNOWN_MODEL_NAME
+        importedModelInfo = self.voiceCardsManager.get(row)
+        if importedModelInfo is not None:
+            name = importedModelInfo.name
+            iconFile = self.voiceCardsManager.getIcon(row)
+            if iconFile is not None:
+                pixmap = QPixmap(iconFile)
+                widget = QLabel()
+                widget.setPixmap(cropCenterScalePixmap(pixmap, VOICE_CARD_SIZE))
+                widget.setToolTip(name)
+        if widget is None:
+            widget = VoiceCardPlaceholderWidget(
+                VOICE_CARD_SIZE, f"{name}<br><br>{DROP_ICON_FILE}"
+            )
+        widget.setToolTip(name)
+        contextMenu = QMenu(widget)
+        deleteAction = QAction("Delete", widget)
+        deleteAction.triggered.connect(lambda: self.takeItem(self.currentRow()))
+        contextMenu.addAction(deleteAction)
+        widget.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
+        widget.customContextMenuRequested.connect(
+            lambda point: contextMenu.exec(widget.mapToGlobal(point))
+        )
+        return widget
 
 
 class VoiceCardPlaceholderWidget(QWidget):
@@ -399,6 +387,7 @@ class VoiceCardPlaceholderWidget(QWidget):
         layout.setContentsMargins(0, 0, 0, 0)
 
         dropHere = QLabel(text)
+        dropHere.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, True)
         dropHere.setTextFormat(Qt.TextFormat.RichText)
         dropHere.setWordWrap(True)
         dropHere.setAlignment(Qt.AlignmentFlag.AlignCenter)
@@ -409,6 +398,9 @@ class VoiceCardPlaceholderWidget(QWidget):
     def sizeHint(self):
         return self.cardSize
 
+    def pixmap(self) -> QPixmap | None:
+        return None
+
 
 def cropCenterScalePixmap(pixmap: QPixmap, targetSize: QSize) -> QPixmap:
     # Original size
